<?php

namespace Bixo\Opportunity\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Opportunity\Http\Requests\OffersRequest;
use Bixo\Opportunity\Interfaces\OffersRepositoryInterface;
use Bixo\Opportunity\Models\Offers;

/**
 * Resource controller class for offers.
 */
class OffersResourceController extends BaseController
{

    /**
     * Initialize offers resource controller.
     *
     * @param type OffersRepositoryInterface $offers
     *
     * @return null
     */
    public function __construct(OffersRepositoryInterface $offers)
    {
        parent::__construct();
        $this->repository = $offers;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Opportunity\Repositories\Criteria\OffersResourceCriteria::class);
    }

    /**
     * Display a list of offers.
     *
     * @return Response
     */
    public function index(OffersRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Opportunity\Repositories\Presenter\OffersPresenter::class)
                ->$function();
        }

        $offers = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('opportunity::offers.names'))
            ->view('opportunity::offers.index', true)
            ->data(compact('offers', 'view'))
            ->output();
    }

    /**
     * Display offers.
     *
     * @param Request $request
     * @param Model   $offers
     *
     * @return Response
     */
    public function show(OffersRequest $request, Offers $offers)
    {

        if ($offers->exists) {
            $view = 'opportunity::offers.show';
        } else {
            $view = 'opportunity::offers.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('opportunity::offers.name'))
            ->data(compact('offers'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new offers.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(OffersRequest $request)
    {

        $offers = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('opportunity::offers.name')) 
            ->view('opportunity::offers.create', true) 
            ->data(compact('offers'))
            ->output();
    }

    /**
     * Create new offers.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(OffersRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $offers                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('opportunity::offers.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('opportunity/offers/' . $offers->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/opportunity/offers'))
                ->redirect();
        }

    }

    /**
     * Show offers for editing.
     *
     * @param Request $request
     * @param Model   $offers
     *
     * @return Response
     */
    public function edit(OffersRequest $request, Offers $offers)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('opportunity::offers.name'))
            ->view('opportunity::offers.edit', true)
            ->data(compact('offers'))
            ->output();
    }

    /**
     * Update the offers.
     *
     * @param Request $request
     * @param Model   $offers
     *
     * @return Response
     */
    public function update(OffersRequest $request, Offers $offers)
    {
        try {
            $attributes = $request->all();

            $offers->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('opportunity::offers.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('opportunity/offers/' . $offers->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('opportunity/offers/' . $offers->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the offers.
     *
     * @param Model   $offers
     *
     * @return Response
     */
    public function destroy(OffersRequest $request, Offers $offers)
    {
        try {

            $offers->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('opportunity::offers.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('opportunity/offers/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('opportunity/offers/' . $offers->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple offers.
     *
     * @param Model   $offers
     *
     * @return Response
     */
    public function delete(OffersRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('opportunity::offers.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('opportunity/offers'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/opportunity/offers'))
                ->redirect();
        }

    }

    /**
     * Restore deleted offers.
     *
     * @param Model   $offers
     *
     * @return Response
     */
    public function restore(OffersRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('opportunity::offers.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/opportunity/offers'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/opportunity/offers/'))
                ->redirect();
        }

    }

}
