<?php

namespace Bixo\Opportunity\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Opportunity\Http\Requests\OpportunitiesRequest;
use Bixo\Opportunity\Interfaces\OpportunitiesRepositoryInterface;
use Bixo\Opportunity\Models\Opportunities;

/**
 * Resource controller class for opportunities.
 */
class OpportunitiesResourceController extends BaseController
{

    /**
     * Initialize opportunities resource controller.
     *
     * @param type OpportunitiesRepositoryInterface $opportunities
     *
     * @return null
     */
    public function __construct(OpportunitiesRepositoryInterface $opportunities)
    {
        parent::__construct();
        $this->repository = $opportunities;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Opportunity\Repositories\Criteria\OpportunitiesResourceCriteria::class);
    }

    /**
     * Display a list of opportunities.
     *
     * @return Response
     */
    public function index(OpportunitiesRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Opportunity\Repositories\Presenter\OpportunitiesPresenter::class)
                ->$function();
        }

        $opportunities = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('opportunity::opportunities.names'))
            ->view('opportunity::opportunities.index', true)
            ->data(compact('opportunities', 'view'))
            ->output();
    }

    /**
     * Display opportunities.
     *
     * @param Request $request
     * @param Model   $opportunities
     *
     * @return Response
     */
    public function show(OpportunitiesRequest $request, Opportunities $opportunities)
    {

        if ($opportunities->exists) {
            $view = 'opportunity::opportunities.show';
        } else {
            $view = 'opportunity::opportunities.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('opportunity::opportunities.name'))
            ->data(compact('opportunities'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new opportunities.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(OpportunitiesRequest $request)
    {

        $opportunities = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('opportunity::opportunities.name')) 
            ->view('opportunity::opportunities.create', true) 
            ->data(compact('opportunities'))
            ->output();
    }

    /**
     * Create new opportunities.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(OpportunitiesRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $opportunities                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('opportunity::opportunities.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('opportunity/opportunities/' . $opportunities->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/opportunity/opportunities'))
                ->redirect();
        }

    }

    /**
     * Show opportunities for editing.
     *
     * @param Request $request
     * @param Model   $opportunities
     *
     * @return Response
     */
    public function edit(OpportunitiesRequest $request, Opportunities $opportunities)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('opportunity::opportunities.name'))
            ->view('opportunity::opportunities.edit', true)
            ->data(compact('opportunities'))
            ->output();
    }

    /**
     * Update the opportunities.
     *
     * @param Request $request
     * @param Model   $opportunities
     *
     * @return Response
     */
    public function update(OpportunitiesRequest $request, Opportunities $opportunities)
    {
        try {
            $attributes = $request->all();

            $opportunities->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('opportunity::opportunities.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('opportunity/opportunities/' . $opportunities->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('opportunity/opportunities/' . $opportunities->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the opportunities.
     *
     * @param Model   $opportunities
     *
     * @return Response
     */
    public function destroy(OpportunitiesRequest $request, Opportunities $opportunities)
    {
        try {

            $opportunities->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('opportunity::opportunities.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('opportunity/opportunities/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('opportunity/opportunities/' . $opportunities->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple opportunities.
     *
     * @param Model   $opportunities
     *
     * @return Response
     */
    public function delete(OpportunitiesRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('opportunity::opportunities.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('opportunity/opportunities'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/opportunity/opportunities'))
                ->redirect();
        }

    }

    /**
     * Restore deleted opportunities.
     *
     * @param Model   $opportunities
     *
     * @return Response
     */
    public function restore(OpportunitiesRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('opportunity::opportunities.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/opportunity/opportunities'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/opportunity/opportunities/'))
                ->redirect();
        }

    }

}
